package com.imcode.forum.sql;

import com.imcode.forum.ForumPost;
import com.imcode.forum.ForumUser;
import com.imcode.forum.ForumThread;

import java.util.Date;
import java.util.SortedSet;

class SqlPost implements ForumPost {

    private String title;
    private ForumUser user;
    private Date dateTime = new Date();
    private String body;
    private int viewCount;
    private SqlRepository repository;
    private final String threadId;
    private SqlThread thread ;
    private final String id;
    private String parentPostId;
    private boolean hidden;

    SqlPost(String id, SqlRepository repository, String threadId, SqlThread thread) {
        this.id = id;
        this.repository = repository;
        this.threadId = threadId;
        this.thread = thread ;
    }

    public void setUser(ForumUser user) {
        this.user = user;
    }

    public String getTitle() {
        return title;
    }

    public ForumUser getUser() {
        return user;
    }

    public void setTitle(String title) {
        this.title = title;
    }

    public Date getDateTime() {
        return (Date) dateTime.clone();
    }

    public String getBody() {
        return body;
    }

    public boolean isHidden() {
        return hidden ;
    }

    public void hide(boolean hidden) {
        getSqlThread().hidePost(id, hidden) ;
        setHidden(hidden) ;
    }

    public void setBody(String body) {
        this.body = body;
    }

    public SortedSet<ForumPost> getReplies() {
        return getSqlThread().getPostReplies(this) ;
    }

    public void incrementViewCount() {
        repository.incrementPostViewCount(id) ;
        viewCount++ ;
    }

    public String toString() {
        return "post "+id+" '"+title+"' created at "+dateTime ;
    }

    public void setDateTime(Date dateTime) {
        this.dateTime = (Date) dateTime.clone() ;
    }

    public void setViewCount(int viewCount) {
        this.viewCount = viewCount;
    }

    public int getViewCount() {
        return viewCount;
    }

    public String createReply(ForumPost forumPost) {
        return getSqlThread().createPostReply(id, forumPost) ;
    }

    public String getId() {
        return id;
    }

    public String getParentPostId() {
        return parentPostId;
    }

    public void setParentPostId(String parentPostId) {
        this.parentPostId = parentPostId;
    }

    public void setHidden(boolean hidden) {
        this.hidden = hidden;
    }

    public ForumThread getThread() {
        return getSqlThread();
    }

    public ForumPost getParentPost() {
        return getSqlThread().getPost(parentPostId) ;
    }

    private SqlThread getSqlThread() {
        if (null == thread) {
            thread = repository.getSqlThread(threadId, null) ;
        }
        return thread;
    }
}
