package com.imcode.forum.sql;

import com.imcode.forum.ForumPost;
import com.imcode.forum.ForumThread;
import com.imcode.forum.Forum;

import java.util.SortedSet;

class SqlThread implements ForumThread {

    private final String id;
    private SqlRepository repository;
    private SqlPostsSet postsSet;
    private final String forumId;
    private Forum forum;

    SqlThread(String id, SqlRepository repository, String forumId, Forum forum
    ) {
        this.id = id;
        this.repository = repository;
        this.forumId = forumId;
        this.forum = forum;
    }

    public ForumPost getTopPost() {
        return getSqlPosts().getTopPost() ;
    }

    public ForumPost getNewestPost() {
        return getSqlPosts().getNewestPost() ;
    }

    public String getId() {
        return id;
    }

    private SqlPostsSet getSqlPosts() {
        if (null == postsSet ) {
            postsSet = repository.getThreadPosts(this);
        }
        return postsSet ;
    }

    public ForumPost getPost(String postId) {
        return getSqlPosts().getPost(postId) ;
    }

    public int getPostCount() {
        return getSqlPosts().size();
    }

    public int getVisiblePostCount() {
        return getSqlPosts().getVisiblePostCount() ;
    }

    public Forum getForum() {
        if (null == forum) {
            forum = repository.getForum(forumId) ;
        }
        return forum ;
    }

    public String toString() {
        return "thread " + id + " with "+getSqlPosts().size() +" posts, top " + getTopPost() + ", and newest "
               + getNewestPost();
    }

    String createPostReply(String postId, ForumPost forumPost) {
        String replyId = repository.createPostReply(id, postId, forumPost);
        postsSet = null ;
        return replyId ;
    }

    SortedSet<ForumPost> getPostReplies(final ForumPost post) {
        return getSqlPosts().getPostReplies(post.getId()) ;
    }

    void incrementPostViewCount(String postId) {
        repository.incrementPostViewCount(postId);
    }

    void hidePost(String postId, boolean hidden) {
        repository.hidePost(postId, hidden) ;
    }
}
