package com.imcode.forum.sql;

import com.imcode.forum.*;
import junit.framework.TestCase;
import org.apache.commons.beanutils.BeanUtils;
import org.apache.commons.dbcp.BasicDataSource;

import java.io.FileInputStream;
import java.util.Properties;
import java.util.SortedSet;

public class SqlRepositoryTest extends TestCase {

    public void testMysql() throws Exception {
        ForumRepository repository = createRepository("mysql.test.properties");
        assertRepository(repository);
    }

    public void testMssql() throws Exception {
        ForumRepository repository = createRepository("mssql.test.properties");
        assertRepository(repository);
    }

    protected ForumRepository createRepository(String propertiesFilename) throws Exception {
        BasicDataSource dataSource = createDataSource(propertiesFilename);
        return new SqlRepository(dataSource);
    }

    protected BasicDataSource createDataSource(String propertiesFilename) throws Exception {
        Properties properties = new Properties();
        properties.load(new FileInputStream(propertiesFilename));
        BasicDataSource dataSource = new BasicDataSource();
        BeanUtils.populate(dataSource, properties);
        return dataSource;
    }

    protected void assertRepository(ForumRepository repo) {
        Forum newForum = repo.createForum("1");
        String forumId = newForum.getId() ;
        Forum forum = repo.getForum(forumId);
        assertNotNull(forum);

        String postTitle = "title";
        String postBody = "body";
        ForumUser postUser = new SimpleForumUser("name", "alias", "emailAddress", "data");
        SimpleForumPost post = new SimpleForumPost(postTitle, postBody, postUser);
        ForumThread newThread = forum.createThread(post);
        assertEquals(forum, newThread.getForum());
        ForumThread thread = forum.getThread(newThread.getId()) ;
        assertNotNull(thread);

        ForumPost topPost = thread.getTopPost();
        assertNotNull(topPost);
        assertNull(topPost.getParentPost()) ;
        assertEquals(postTitle, topPost.getTitle()) ;
        assertEquals(postBody, post.getBody());
        assertEquals(postUser, post.getUser());
        assertEquals(post.getDateTime().getTime() / 1000, topPost.getDateTime().getTime() / 1000) ;
        assertEquals(post.isHidden(), topPost.isHidden()) ;

        String replyTitle = "reply title";
        String replyBody = "reply body";
        ForumUser replyUser = new SimpleForumUser("reply name", "reply alias", "reply emailAddress", "reply data");
        SimpleForumPost newReply = new SimpleForumPost(replyTitle, replyBody, replyUser);
        String replyId = topPost.createReply(newReply);
        assertNotNull(replyId) ;
        ForumPost reply = thread.getPost(replyId) ;
        assertNotNull(reply) ;
        assertTrue(topPost.getReplies().contains(reply)) ;
        assertEquals(replyTitle, reply.getTitle()) ;
        assertEquals(replyBody, reply.getBody()) ;
        assertEquals(replyUser, reply.getUser()) ;

        String replyReplyId = reply.createReply(newReply) ;
        ForumPost replyReply = thread.getPost(replyReplyId) ;
        assertNotNull(replyReply) ;
        assertTrue(reply.getReplies().contains(replyReply)) ;
        replyReply.hide(true);
        assertTrue(replyReply.isHidden()) ;
        assertTrue(thread.getPost(replyReplyId).isHidden()) ;

        replyReply.incrementViewCount();
        assertEquals(1, thread.getPost(replyReply.getId()).getViewCount());

        int postCount = thread.getPostCount();
        assertEquals(3, postCount) ;

        SortedSet<ForumPost> replies = topPost.getReplies();
        assertEquals(1, replies.size()) ;
        assertEquals(topPost.getId(), replies.last().getParentPost().getId()) ;

        assertFalse(0 == forum.searchPosts(new String[] { "reply" }).size()) ;
        assertFalse(0 == forum.searchThreads(new String[] { "reply" }).size()) ;
    }
}
